import{aJ as t}from"./index-WOIPm_2l.js";function r(){const e=localStorage.getItem("uiTheme");return!e||!["PurpleTheme","PurpleThemeDark"].includes(e)?(localStorage.setItem("uiTheme","PurpleTheme"),"PurpleTheme"):e}const i={Sidebar_drawer:!0,Customizer_drawer:!1,mini_sidebar:!1,uiTheme:r(),inputBg:!1},s=t({id:"customizer",state:()=>({Sidebar_drawer:i.Sidebar_drawer,Customizer_drawer:i.Customizer_drawer,mini_sidebar:i.mini_sidebar,fontTheme:"Poppins",uiTheme:i.uiTheme,inputBg:i.inputBg,viewMode:localStorage.getItem("viewMode")||"bot",chatSidebarOpen:!1}),getters:{},actions:{SET_SIDEBAR_DRAWER(){this.Sidebar_drawer=!this.Sidebar_drawer},SET_MINI_SIDEBAR(e){this.mini_sidebar=e},SET_FONT(e){this.fontTheme=e},SET_UI_THEME(e){this.uiTheme=e,localStorage.setItem("uiTheme",e)},SET_VIEW_MODE(e){this.viewMode=e,localStorage.setItem("viewMode",e)},TOGGLE_CHAT_SIDEBAR(){this.chatSidebarOpen=!this.chatSidebarOpen},SET_CHAT_SIDEBAR(e){this.chatSidebarOpen=e}}});export{s as useCustomizerStore};
