import{C as t}from"./Chat-CaT5e3YI.js";import{_ as o,c as r,b as a,o as e}from"./index-WOIPm_2l.js";import"./customizer-mKzhlNaN.js";import"./MessageList-DFdDzCzJ.js";import"./toast-CbgsPrO_.js";import"./confirmDialog-n5rmADX5.js";import"./StyledMenu.vue_vue_type_style_index_0_lang-CHdWoe5_.js";import"./ProviderConfigDialog-DqlrQkP5.js";import"./AstrBotConfig-Dr_QRx4-.js";import"./inputValue-De_-AW4_.js";import"./PersonaForm-DXaTrPIk.js";import"./useRecording-C6fOA4L_.js";const p={class:"chat-container"},c={__name:"ChatPage",setup(m){return(s,i)=>(e(),r("div",p,[a(t)]))}},B=o(c,[["__scopeId","data-v-a70ec87e"]]);export{B as default};
