.base-move-target-node[data-v-a4cf771e]{width:100%}.folder-item[data-v-a4cf771e]{min-height:36px}.expand-btn[data-v-a4cf771e]{margin-right:4px}.expand-placeholder[data-v-a4cf771e]{width:28px;flex-shrink:0}.persona-form-card[data-v-c6bd3d5e]{border-radius:12px;overflow:hidden}.persona-form-content[data-v-c6bd3d5e]{max-height:min(78vh,760px);overflow-y:auto}.persona-form-title[data-v-c6bd3d5e]{line-height:1.3}.persona-form-actions[data-v-c6bd3d5e]{position:sticky;bottom:0;z-index:2;background:rgb(var(--v-theme-surface));border-top:1px solid rgba(var(--v-border-color),var(--v-border-opacity))}.selected-config-area[data-v-c6bd3d5e]{margin-left:32px}.persona-form-layout[data-v-c6bd3d5e]{align-items:flex-start}.tools-selection[data-v-c6bd3d5e],.skills-selection[data-v-c6bd3d5e]{max-height:300px;overflow-y:auto}.v-virtual-scroll[data-v-c6bd3d5e]{padding-bottom:16px}@media(max-width:600px){.persona-form-card-mobile[data-v-c6bd3d5e]{border-radius:0}.persona-form-content[data-v-c6bd3d5e]{max-height:calc(100vh - 128px);padding:16px!important}.persona-basic-col[data-v-c6bd3d5e],.persona-panels-col[data-v-c6bd3d5e]{padding-top:0!important}.persona-form-title[data-v-c6bd3d5e]{font-size:1.15rem!important;padding:12px 16px!important}.selected-config-area[data-v-c6bd3d5e]{margin-left:0}.tools-selection[data-v-c6bd3d5e],.skills-selection[data-v-c6bd3d5e]{max-height:38vh}.persona-form-actions[data-v-c6bd3d5e]{padding:12px 16px!important;gap:8px}.persona-form-actions .v-btn[data-v-c6bd3d5e]{min-width:0}}
