import{e as n,j as l,w as t,b as s,K as r,ag as c,s as a,C as p,af as d,o as i}from"./index-WOIPm_2l.js";const C=n({inheritAttrs:!1,__name:"StyledMenu",props:{closeOnContentClick:{type:Boolean,default:!0}},setup(u){return(e,f)=>(i(),l(d,p(e.$attrs,{"close-on-content-click":e.closeOnContentClick}),{activator:t(({props:o})=>[a(e.$slots,"activator",{props:o})]),default:t(()=>[s(r,{class:"styled-menu-card",elevation:"8",rounded:"lg"},{default:t(()=>[s(c,{density:"compact",class:"styled-menu-list pa-1"},{default:t(()=>[a(e.$slots,"default")]),_:3})]),_:3})]),_:3},16,["close-on-content-click"]))}});export{C as _};
