import{e,j as a,w as t,ay as o,b as s,z as n,az as r,o as c}from"./index-WOIPm_2l.js";const f=e({__name:"BlankLayout",setup(p){return(u,_)=>(c(),a(o,null,{default:t(()=>[s(n(r))]),_:1}))}});export{f as default};
