import{e as r,c,b as e,w as o,a1 as _,T as i,j as l,aO as d,o as a,_ as m}from"./index-WOIPm_2l.js";const p={class:"kb-container"},u=r({__name:"index",setup(f){return(t,k)=>{const n=_("router-view");return a(),c("div",p,[e(n,null,{default:o(({Component:s})=>[e(i,{name:"kb-fade",mode:"out-in"},{default:o(()=>[(a(),l(d(s),{key:t.$route.fullPath}))]),_:2},1024)]),_:1})])}}}),x=m(u,[["__scopeId","data-v-8a796453"]]);export{x as default};
