.provider-config-dialog[data-v-55280f9c]{height:calc(100vh - 100px);display:flex;flex-direction:column}.provider-config-dialog.mobile-dialog[data-v-55280f9c]{height:100vh}.provider-sources-column[data-v-55280f9c]{overflow-y:auto;background-color:var(--v-theme-surface)}.provider-config-column[data-v-55280f9c]{background-color:var(--v-theme-background)}.border[data-v-55280f9c]{border:1px solid rgba(var(--v-border-color),var(--v-border-opacity))}.mobile-content[data-v-55280f9c]{padding:0 8px 8px!important;height:calc(100vh - 64px)!important;max-height:none!important}.mobile-layout[data-v-55280f9c]{display:flex;flex-direction:column;height:100%;gap:16px}.mobile-sources[data-v-55280f9c]{width:100%!important;min-width:100%!important;border-right:none!important;border-bottom:1px solid rgba(var(--v-border-color),var(--v-border-opacity));max-height:40vh;overflow-y:auto}.mobile-config[data-v-55280f9c]{flex:1;overflow-y:auto;min-width:100%!important}@media(max-width:768px){.provider-config-dialog[data-v-55280f9c] .v-card-title{padding:12px 16px!important}.provider-config-dialog[data-v-55280f9c] .v-card-title .text-h2{font-size:1.5rem!important}}
