DEERFLOW_PROVIDER_TYPE = "deerflow"
DEERFLOW_THREAD_ID_KEY = "deerflow_thread_id"
DEERFLOW_SESSION_PREFIX = "deerflow-ephemeral"
DEERFLOW_AGENT_RUNNER_PROVIDER_ID_KEY = "deerflow_agent_runner_provider_id"
