.v-select__selection-text{font-size:12px}.config-drawer-overlay{align-items:stretch;justify-content:flex-end}.config-drawer-card{width:clamp(320px,60vw,820px);height:calc(100vh - 32px);display:flex;flex-direction:column;margin:16px}.config-drawer-header{display:flex;align-items:center;justify-content:space-between;padding:16px 20px 12px}.config-drawer-content{flex:1;overflow-y:auto;padding:16px 16px 24px}
