.kb-container[data-v-8a796453]{width:100%;height:100%;position:relative}.kb-fade-enter-active[data-v-8a796453],.kb-fade-leave-active[data-v-8a796453]{transition:opacity .3s ease,transform .3s ease}.kb-fade-enter-from[data-v-8a796453]{opacity:0;transform:translateY(10px)}.kb-fade-leave-to[data-v-8a796453]{opacity:0;transform:translateY(-10px)}
