from __future__ import annotations

from .extractor import extract_quoted_message_images, extract_quoted_message_text

__all__ = [
    "extract_quoted_message_text",
    "extract_quoted_message_images",
]
