<template>
    <div style="display: flex; justify-content: center; align-items: center; margin-top: 16px; gap: 12px;">
        <span style="font-size: 13px; color: #666; font-style: italic;;">Deployed on</span>
        <a href="https://www.rainyun.com/NjY3OTQ1_"><img src="https://www.rainyun.com/img/logo.d193755d.png" width="50" alt="Rainyun Logo"></a>
        
    </div>
</template>