import{useCustomizerStore as m}from"./customizer-mKzhlNaN.js";import{_ as u,c as p,a as t,t as n,b as r,w as a,d as l,V as c,u as _,o as f}from"./index-WOIPm_2l.js";const h={name:"AboutPage",setup(){const{tm:e}=_("features/about");return{tm:e}},methods:{useCustomizerStore:m,open(e){window.open(e,"_blank")}}},x={style:{display:"flex","flex-direction":"column",height:"100%"}},y={style:{"flex-grow":"1",display:"flex","align-items":"center","justify-content":"center","flex-direction":"column"}},v={style:{"text-align":"center","max-width":"600px"}},g={class:"font-weight-bold"},b={class:"text-subtitle-1",style:{color:"var(--v-theme-secondaryText)"}},B={style:{"margin-top":"20px",display:"flex","justify-content":"center"}};function w(e,o,A,s,k,i){return f(),p("div",x,[t("div",y,[t("div",v,[t("h1",g,n(s.tm("hero.title")),1),t("p",b,n(s.tm("hero.subtitle")),1),t("div",B,[r(c,{onClick:o[0]||(o[0]=d=>i.open("https://github.com/AstrBotDevs/AstrBot")),color:"primary",variant:"tonal",size:"small","prepend-icon":"mdi-star"},{default:a(()=>[l(n(s.tm("hero.starButton")),1)]),_:1}),r(c,{class:"ml-4",onClick:o[1]||(o[1]=d=>i.open("https://github.com/AstrBotDevs/AstrBot/issues")),color:"secondary",size:"small",variant:"tonal","prepend-icon":"mdi-comment-question"},{default:a(()=>[l(n(s.tm("hero.issueButton")),1)]),_:1})])])])])}const z=u(h,[["render",w]]);export{z as default};
