import{C as o}from"./Chat-CaT5e3YI.js";import{useCustomizerStore as r}from"./customizer-mKzhlNaN.js";import{_ as i,j as a,w as s,a as t,b as m,z as c,ay as p,o as n}from"./index-WOIPm_2l.js";import"./MessageList-DFdDzCzJ.js";import"./toast-CbgsPrO_.js";import"./confirmDialog-n5rmADX5.js";import"./StyledMenu.vue_vue_type_style_index_0_lang-CHdWoe5_.js";import"./ProviderConfigDialog-DqlrQkP5.js";import"./AstrBotConfig-Dr_QRx4-.js";import"./inputValue-De_-AW4_.js";import"./PersonaForm-DXaTrPIk.js";import"./useRecording-C6fOA4L_.js";const _={style:{height:"100%",width:"100%",display:"flex","flex-direction":"column","align-items":"center","justify-content":"center"}},d={id:"container"},h={__name:"ChatBoxPage",setup(u){const e=r();return(f,l)=>(n(),a(p,{theme:c(e).uiTheme,style:{height:"100%",width:"100%"}},{default:s(()=>[t("div",_,[t("div",d,[m(o,{"chatbox-mode":!0})])])]),_:1},8,["theme"]))}},N=i(h,[["__scopeId","data-v-5eb594d9"]]);export{N as default};
