.actions-wrapper{display:flex;justify-content:flex-end;gap:8px}.action-button{border-radius:8px;font-weight:500}.monaco-editor-container{height:500px;border-radius:8px;overflow:hidden;box-shadow:0 2px 8px #0000000d}.conversation-messages-container{max-height:500px;overflow-y:auto;padding:8px;border-radius:8px;background-color:#f9f9f9}.v-theme--dark .conversation-messages-container{background-color:#1e1e1e}.conversation-detail-card{max-height:90vh;display:flex;flex-direction:column}.text-truncate{display:inline-block;white-space:nowrap;overflow:hidden;text-overflow:ellipsis}@keyframes fadeIn{0%{opacity:0;transform:translateY(10px)}to{opacity:1;transform:translateY(0)}}
