.subagent-page[data-v-b10644d2]{padding:24px;max-width:1200px;margin:0 auto}.subagent-panels[data-v-b10644d2] .v-expansion-panel-text__wrapper{padding:16px 16px 42px}.gap-2[data-v-b10644d2]{gap:8px}.gap-4[data-v-b10644d2]{gap:16px}
