.filter-controls[data-v-4c845fdd]{display:flex;flex-wrap:wrap;gap:8px;margin-bottom:8px;margin-left:20px}[data-v-4c845fdd] .console-log-line{display:block;margin-bottom:2px;font-family:SFMono-Regular,Menlo,Monaco,Consolas,var(--astrbot-font-cjk-mono),monospace;font-size:12px;white-space:pre-wrap}[data-v-4c845fdd] .fade-in{animation:fadeIn-4c845fdd .3s}@keyframes fadeIn-4c845fdd{0%{opacity:0}to{opacity:1}}
