.draggable-item[data-v-9674b3df]{cursor:move;border:1px solid rgba(var(--v-border-color),var(--v-border-opacity));border-radius:4px;background-color:rgba(var(--v-theme-surface));transition:all .2s}.draggable-item[data-v-9674b3df]:hover{background-color:rgba(var(--v-theme-primary),.1);border-color:rgba(var(--v-theme-primary),.3)}.custom-list[data-v-9674b3df]{min-height:200px;border:1px dashed rgba(var(--v-border-color),var(--v-border-opacity));border-radius:4px;padding:8px}.v-list-item[data-v-bd3b5867]{border-bottom:1px solid rgba(0,0,0,.08)}.v-list-item[data-v-bd3b5867]:last-child{border-bottom:none}.non-interactive-chip[data-v-bd3b5867]{pointer-events:none;cursor:default}.non-interactive-chip[data-v-bd3b5867]:hover{box-shadow:none!important}
