.v-data-table[data-v-85960659] .v-data-table__td{padding:8px 16px!important;vertical-align:middle!important}code[data-v-85960659]{background-color:#0000000d;padding:2px 6px;border-radius:4px;font-size:12px}.transfer-list[data-v-85960659]{max-height:280px;overflow-y:auto;border:1px solid rgba(0,0,0,.12);border-radius:4px}.transfer-item[data-v-85960659]{cursor:pointer;transition:background-color .15s}.transfer-item[data-v-85960659]:hover{background-color:#0000000a}
