.styled-menu-card{min-width:100px;width:fit-content;border:1px solid rgba(var(--v-theme-primary),.15)!important;background:rgba(var(--v-theme-surface),.98)!important;-webkit-backdrop-filter:blur(10px);backdrop-filter:blur(10px)}.styled-menu-list{background:transparent!important}.styled-menu-item{margin:2px 0;transition:all .2s ease;border-radius:6px}.styled-menu-item:hover{background:rgba(var(--v-theme-primary),.08)!important}.styled-menu-item-active{background:rgba(var(--v-theme-primary),.15)!important;font-weight:500}.styled-menu-item-active:hover{background:rgba(var(--v-theme-primary),.2)!important}.v-theme--PurpleThemeDark .styled-menu-card{background:rgba(var(--v-theme-surface),.98)!important;border:1px solid rgba(var(--v-theme-primary),.2)!important}.v-theme--PurpleThemeDark .styled-menu-item:hover{background:rgba(var(--v-theme-primary),.12)!important}.v-theme--PurpleThemeDark .styled-menu-item-active{background:rgba(var(--v-theme-primary),.2)!important}.v-theme--PurpleThemeDark .styled-menu-item-active:hover{background:rgba(var(--v-theme-primary),.25)!important}
